/*
===============================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

-------------------------------------------------------------------------------
    Filename: uvi_api.c  Version: 1.1  Modification Date: 00/03/21 10:14:23
-------------------------------------------------------------------------------
    Description: API library functions for the Universe driver for Linux
-------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  ---------------------------------------------------
    21-MAR-2000  JW   1.1   Created.
    23-SEP-2000  SL         Added DMA, Mailbox, and slave
    19-JAN-2001  SL         Added servicing VME interrupts
    14-FEB-2001  SL         Removed threads from Mailbox, DMA, and VME
                            interrupt functions.
===============================================================================
*/

#ident "@(#)uvi_api.c 1.1 00/06/27 VMIC"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <asm/page.h>
#include <fcntl.h>
#include <asm/errno.h>
#include <errno.h>
#include <pthread.h>
#include <linux/param.h>

#include "uvi_api.h"


/******************************************************************************
*
*  FUNCTION:   VmeOpen
*
*  PURPOSE:    Open the Universe driver and return a file descriptor
*  PARAMETERS: None
*  RETURNS:    Valid file descriptor from open(2) call if no errors
*              UVI_NO_DEV_FOUND if the special device file is not found
*              UVI_OS_ERROR if the open(2) call fails
*  GLOBALS:    None
*
******************************************************************************/

int 
VmeOpen()
{
    int fdesc;    /* File description returned from open(2) call  */
	
    /* Make sure the device file is present */
    if( access( DEVICE, F_OK ) != 0 )
    {
        return( UVI_NODEV_FOUND );
    }
        
    /* Open the device driver */
    fdesc = open( DEVICE, O_RDWR );
    if( fdesc < 0 )
    {
        return( UVI_OS_ERROR );
    }
        
    return fdesc;

}  /* end of VmeOpen() */


/******************************************************************************
*
*  FUNCTION:   VmeGetPointer 
*
*  PURPOSE:    Map a window of VMEbus address space into a user application
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              VmeBusAddr (I): Base address of VME window
*              AddrMod (I): Address modifier of VME window
*              ReqWndSize (I): Size of VME window in bytes
*              Handle (IO): Points to a Handle for this VME window
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_NULL_PTR if the Handle pointer is NULL
*              UVI_OS_ERROR if the ioctl(2) fails
*              UVI_MAP_ERROR if the mmap(2) fails
*  GLOBALS:    None
*
******************************************************************************/

int 
VmeGetPointer( UVI_UINT32 fdesc, UVI_UINT32 VmeBusAddr, UVI_UINT32 AddrMod, 
    UVI_UINT32 ReqWndSize, UVIHANDLE *Handle )
{
    unsigned int temp_int;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }
        
    /* Check the handle */
    if( Handle == (UVIHANDLE *) NULL )
    {
        return( UVI_NULL_PTR );
    }

    Handle->MappedUvi = (void *)NULL;
    Handle->MappedSize = 0;
    Handle->ptr = (void *)NULL;
    Handle->fd = fdesc;

    /* Call the driver to get the physical PCI address of the VME window */
    Handle->Parameters.ReqWndSize = ReqWndSize;
    Handle->Parameters.AddrMod = AddrMod;
    Handle->Parameters.VmeBusAddr = VmeBusAddr;
    Handle->Parameters.SysModeFlag = UVI_USER;
    Handle->Parameters.PhysAddr = 0x0;
    if ( ioctl(fdesc, IOCTL_UVI_GET_WINDOW, &( Handle->Parameters ) ) != 0 )
    {
	return( UVI_OS_ERROR );
    }

    /* Map the VME window into user space */
    Handle->MappedUvi = mmap( 0, ReqWndSize, 
			PROT_READ | PROT_WRITE, 
			MAP_SHARED, 
			Handle->fd, 
			Handle->Parameters.PhysAddr );
 
    if ( Handle->MappedUvi == (void *)-1 )
    {
	return( UVI_MAP_ERROR );
    }

    Handle->MappedSize = ReqWndSize;

    /* mmap(2) returned a pointer rounded down to the nearest PAGE_SIZE 
       boundary.  Now adjust for page size */
    temp_int = (unsigned int)Handle->MappedUvi;
    temp_int += ( VmeBusAddr % PAGE_SIZE );
    Handle->ptr = (void *)temp_int;

    return( UVI_SUCCESS );

}  /* end of VmeGetPointer() */


/******************************************************************************
*
*  FUNCTION:   VmeReleasePointer
*
*  PURPOSE:    Release a mapped VME address space from a user application
*  PARAMETERS: Handle (IO): Points to a Handle for this VME window
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_MAP_ERROR if the munmap(2) fails
*              UVI_OS_ERROR if the ioctl(2) fails
*  GLOBALS:    None
*
******************************************************************************/

int 
VmeReleasePointer( UVIHANDLE *Handle )
{
    /* Check the handle */
    if( Handle == (UVIHANDLE *) NULL )
    {
        return( UVI_NULL_PTR );
    }

    errno = 0;
    
    /* Unmap the VME window from user space */
    if (munmap(Handle->MappedUvi, Handle->MappedSize) != 0)
    {
	return( UVI_MAP_ERROR );
    }

    if ( ioctl( Handle->fd, IOCTL_UVI_CLOSE_WINDOW, &( Handle->Parameters ) ) != 0 )
    {
	return( UVI_OS_ERROR );
    }

    return( UVI_SUCCESS );

}  /* end of VmeReleasePointer() */


/******************************************************************************
*
*  FUNCTION:   VmeClose
*
*  PURPOSE:    Closes the Universe device driver
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*  RETURNS:    UVI_SUCCESS if no error occurs
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails              
*  GLOBALS:    None
*
******************************************************************************/

int 
VmeClose( UVI_UINT32 fdesc )
{
    if ( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }

    if ( close( fdesc ) != 0 )
    {
	    return( UVI_OS_ERROR );
    }

    return( UVI_SUCCESS );

}  /* end of VmeClose() */


/******************************************************************************
*
*  FUNCTION:   VmeSetDebugFlags
*
*  PURPOSE:    Set or clear flags to enable/disable the driver's debug messages
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              Flags (I): Bitmask defined in error_codes.h
*  RETURNS:    UVI_SUCCESS if no error occurs
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails              
*  GLOBALS:    None
*
******************************************************************************/

int 
VmeSetDebugFlags( UVI_UINT32 fdesc, UVI_UINT32 Flags )
{
    /* Validate the file descriptor */
    if ( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }

    /* Set the debug flags */
    if( ioctl( fdesc, IOCTL_UVI_SET_DEBUG_FLAGS, Flags ) != 0 )
    {
	    return( UVI_OS_ERROR );
    }
    
    return( UVI_SUCCESS );

}  /* end of VmeSetDebugFlags() */


/******************************************************************************
*
*  FUNCTION:   VmeGetDebugFlags
*
*  PURPOSE:    Get the flags that enable/disable the driver's debug messages
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              Flags (IO): Bitmask defined in error_codes.h
*  RETURNS:    UVI_SUCCESS if no error occurs
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_NULL_PTR if the Flags pointer is NULL
*              UVI_OS_ERROR if the ioctl(2) fails              
*  GLOBALS:    None
*
******************************************************************************/

int 
VmeGetDebugFlags( UVI_UINT32 fdesc, UVI_UINT32 *Flags )
{
    /* Validate the file descriptor */
    if ( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }

    /* Validate the Flags pointer */
    if( Flags == (UVI_UINT32 *) NULL )
    {
        return( UVI_NULL_PTR );
    }
    
    /* Get the debug flags */
    if( ioctl( fdesc, IOCTL_UVI_GET_DEBUG_FLAGS, Flags ) != 0 )
    {
	    return( UVI_OS_ERROR );
    }
    
    return( UVI_SUCCESS );

}  /* end of VmeGetDebugFlags() */


/******************************************************************************
*
*  FUNCTION:   VmeProbeAddress
*
*  PURPOSE:    Probe a VMEbus address to check for bus error
*  PARAMETERS: name: (IO) description
*  RETURNS:    UVI_SUCCESS if no errors, and the probe did not bus error
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_BAD_PARAMETER_3 if the address modifier is invalid
*              UVI_BAD_PARAMETER_4 if the width is invalid
*  GLOBALS:    
*
******************************************************************************/

int 
VmeProbeAddress( UVI_UINT32 fdesc, UVI_UINT32 Address, UVI_UINT32 Modifier, 
    UVI_UINT32 Width )
{
    struct uviVmeWindowParameters window;
    
    /* Validate the file descriptor */
    if ( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }

    /* Validate the address modifier */
    switch( Modifier )
    {
        case V_A16UD: /* Fall thru */
        case V_A16SD: /* Fall thru */
        case V_A24UD: /* Fall thru */
        case V_A24SD: /* Fall thru */
        case V_A32UD: /* Fall thru */
        case V_A32SD:
            break;
            
        default:
            return( UVI_BAD_PARAMETER_3 );
    }

    /* Validate the width */
    switch( Width )
    {
        case UVI_BYTE:  /* Fall thru */
        case UVI_WORD:  /* Fall thru */
        case UVI_LWORD: /* Fall thru */
            break;

        default:
            return( UVI_BAD_PARAMETER_4 );
    }

    /* Pass the width parameter in this member */
    window.ReqWndSize = Width;
    
    /* Do the probe */
    window.AddrMod    = Modifier;
    window.VmeBusAddr = Address;
    if( ioctl( fdesc, IOCTL_UVI_PROBE_ADDRESS, &window ) != 0 )
    switch( errno )
    {
        case EIO:
            return( UVI_BERR );
            break;
            
        default:
            return( UVI_OS_ERROR );
    }
    
    return( UVI_SUCCESS );

}  /* end of VmeProbeAddress() */


/******************************************************************************
*
*  FUNCTION:   VmeReleaseDma
*
*  PURPOSE:    Release control of the DMA resource
*  PARAMETERS: fdesc (I): File descriptor from open
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*  GLOBALS:    None
*
******************************************************************************/

int 
VmeReleaseDma( UVI_UINT32 fdesc )
{
    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }
        
    if ( ioctl(fdesc, IOCTL_UVI_RELEASE_DMA, 0 ) != 0 )
    {
        /* if resource busy return busy */
	return( UVI_OS_ERROR );
    }

    return( UVI_SUCCESS );

}  /* end of VmeReleaseDma() */


/******************************************************************************
*
*  FUNCTION:   VmeReqDma
*
*  PURPOSE:    Request use of the tundra DMA resource  
*  PARAMETERS: fdesc (I): File descriptor from open
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*  GLOBALS:    None
*
******************************************************************************/

int 
VmeReqDma( UVI_UINT32 fdesc )
{
    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }
        
    /* IOCTL to use universe DMA resource.
       driver_entry calls uviRequestUseDma(), if request returns not
       busy the DMA resource has been locked.
    */
    if ( ioctl(fdesc, IOCTL_UVI_REQ_DMA, 0 ) != 0 )
    {
        /* if resource busy return busy */
	return( UVI_DMA_BUSY );
    }
    return( UVI_SUCCESS );

}  /* end of VmeReqDma() */


/******************************************************************************
*
*  FUNCTION:   VmeDmaIO
*
*  PURPOSE:    Perform DMA between local buffer and VME
*  PARAMETERS: fdesc (I): File descriptor from open
*              VmeBusAddr (I): VME Base address for DMA
*              AddrMod (I): Address modifier
*              DmaSize (I): Size of transfer in bytes
*              DataWidth (I): Vme data width
*              ReadWrite (I): Dma read write flag
*              DmaOffset (I): Offset from local DMA_BUFF_BASE 
*              *DmaCompStat (I/O): Dma completion status
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*  GLOBALS:    None
*
******************************************************************************/

int 
VmeDmaIO( UVI_UINT32 fdesc, UVI_UINT32 VmeBusAddr, UVI_UINT32 AddrMod, 
          UVI_UINT32 DmaSize, UVI_UINT32 DataWidth, UVI_BOOL ReadWrite, 
          UVI_UINT32 DmaOffset, UVI_UINT32 * DmaCompStat )
{
    struct uviVmeDmaParameters dmaparams;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }
        
    dmaparams.Fdesc = fdesc;
    dmaparams.DmaCompStat = 0;
    dmaparams.DmaRdWr = ReadWrite;
    dmaparams.DmaSize = DmaSize;
    dmaparams.AddrMod = AddrMod;
    dmaparams.VmeBusAddr = VmeBusAddr;
    dmaparams.DataWidth = DataWidth;
    dmaparams.DmaOffset = DmaOffset;

    if ( ioctl(fdesc, IOCTL_UVI_DMAIO, &dmaparams ) != 0 )
    {
        return( UVI_OS_ERROR );
    }

    *DmaCompStat = dmaparams.DmaCompStat;

    return( UVI_SUCCESS );

}  /* end of VmeDmaIO() */


/******************************************************************************
*
*  FUNCTION:   OpenVmeSlave
*
*  PURPOSE:    Open a VME slave window to local DRAM
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              VmeBusAddr (I): VME Base address for slave window
*              SuperUser (I): Supervisory, Nonpriviledge, or Both
*              PgmData (I): Program, Data, or Both
*              AddSpace (I): A16, A24, or A32 address space
*              WinSize (I): Size in bytes of slave window (4k or 64k granularity)
*              Offset (I): Offset from local SLAVE_BUFF_BASE 
*              WinNum (I): One of Eight slave windows 
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*  GLOBALS:    None
*
*  NOTE:       Slave windows 0 & 4 have 4k granularity, 1-3,5-7 have 64k granularity
*
******************************************************************************/

int 
OpenVmeSlave( UVI_UINT32 fdesc, UVI_UINT32 VmeBusAddr, UVI_UINT32 SuperUser,
              UVI_UINT32 PgmData, UVI_UINT32 AddSpace, UVI_UINT32 WinSize, 
              UVI_UINT32 Offset, UVI_UINT32 WinNum )
{
    struct uviVmeSlaveWinParameters slvwinparams;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }
        
    slvwinparams.SysModeFlag = UVI_USER; 
    slvwinparams.Fdesc = fdesc;
    slvwinparams.VmeBusAddr = VmeBusAddr;
    slvwinparams.SuperUser = SuperUser;
    slvwinparams.PgmData = PgmData;
    slvwinparams.AddSpace = AddSpace;
    slvwinparams.WinSize = WinSize;
    slvwinparams.Offset = Offset;
    slvwinparams.WinNum = WinNum;

    if ( ioctl(fdesc, IOCTL_UVI_SLAVE_OPEN, &slvwinparams ) != 0 )
    {
	return( UVI_OS_ERROR );
    }

    return( UVI_SUCCESS );

} /* end ConfigVmeSlave */


/******************************************************************************
*
*  FUNCTION:   CloseVmeSlave
*
*  PURPOSE:    Close a VME slave window to local DRAM
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              WinNum (I): One of Eight slave windows 
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*  GLOBALS:    None
*
******************************************************************************/

int 
CloseVmeSlave( UVI_UINT32 fdesc, UVI_UINT32 WinNum )
{
    struct uviVmeSlaveCloseParams slvwinparams;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }
        
    slvwinparams.SysModeFlag = UVI_USER; 
    slvwinparams.Fdesc = fdesc;
    slvwinparams.WinNum = WinNum;

    if ( ioctl(fdesc, IOCTL_UVI_SLAVE_CLOSE, &slvwinparams ) != 0 )
    {
	return( UVI_OS_ERROR );
    }

    return( UVI_SUCCESS );

} /* end CloseVmeSlave */


/******************************************************************************
*
*  FUNCTION:   RequestVmeInt 
*
*  PURPOSE:    Request VME interrupt level/vector.
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              VmeLevel (I): One of seven VME interrupt levels 
*              VmeVector (I): VME interrupt vector
*              
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*              UVI_BAD_PARAMETER_1 if level is out of range
*              UVI_BAD_PARAMETER_2 if vector is out of range
*  GLOBALS:    None
*
******************************************************************************/

int 
RequestVmeInt( UVI_UINT32 fdesc, UVI_UINT8 VmeLevel, UVI_UINT8 VmeVector )
{

    struct uviInterruptParameters IntParams;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }

    IntParams.Fdesc = fdesc;
    IntParams.IntLevel = VmeLevel;
    IntParams.IntVector = VmeVector;
    IntParams.IntStatus = 0;

    if ( ioctl(fdesc, IOCTL_UVI_REQ_VME_INT, &IntParams ) != 0 )
    {
	return( UVI_OS_ERROR );
    }

    return( IntParams.IntStatus );

} /* end RequestVmeInt */


/******************************************************************************
*
*  FUNCTION:   ReleaseVmeInt 
*
*  PURPOSE:    Release VME interrupt level/vector.
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              VmeLevel (I): One of seven VME interrupt levels 
*              VmeVector (I): VME interrupt vector
*              
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*              UVI_VMEINT_WAITING if sleeping task waiting for interrupt (issue cancel first)
*              UVI_VMEINT_NOT_USER if level/vector not user owned
*              UVI_BAD_PARAMETER_1 if level is out of range
*              UVI_BAD_PARAMETER_2 if vector is out of range
*  GLOBALS:    None
*
******************************************************************************/

int 
ReleaseVmeInt( UVI_UINT32 fdesc, UVI_UINT8 VmeLevel, UVI_UINT8 VmeVector )
{

    struct uviInterruptParameters IntParams;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }

    IntParams.Fdesc = fdesc;
    IntParams.IntLevel = VmeLevel;
    IntParams.IntVector = VmeVector;

    if ( ioctl(fdesc, IOCTL_UVI_RELEASE_VME_INT, &IntParams ) != 0 )
    {
	return( UVI_OS_ERROR );
    }

    return( IntParams.IntStatus );

} /* end ReleaseVmeInt */


/******************************************************************************
*
*  FUNCTION:   CancelWaitVmeInt 
*
*  PURPOSE:    Release VME interrupt level/vector.
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              VmeLevel (I): One of seven VME interrupt levels 
*              VmeVector (I): VME interrupt vector
*              
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*              UVI_VMEINT_NOTWAITING if there is NO sleeping task waiting for interrupt
*              UVI_VMEINT_NOT_USER if the level/vector is not user owned
*  GLOBALS:    None
*
******************************************************************************/

int 
CancelWaitVmeInt( UVI_UINT32 fdesc, UVI_UINT8 VmeLevel, UVI_UINT8 VmeVector )
{

    struct uviInterruptParameters IntParams;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }

    IntParams.Fdesc = fdesc;
    IntParams.IntLevel = VmeLevel;
    IntParams.IntVector = VmeVector;

    if ( ioctl(fdesc, IOCTL_UVI_CANCEL_VME_INT, &IntParams ) != 0 )
    {
	return( UVI_OS_ERROR );
    }

    return( IntParams.IntStatus );

} /* end CancelVmeInt */



/******************************************************************************
*
*  FUNCTION:   VmeWaitInt 
*
*  PURPOSE:    Request servicing one of seven VME interrupt levels.
*              If request is successfull, initiate thread that 
*              loops forever calling the wait for interrupt that
*              sleeps waiting for the interrupt. If the process wakes up
*              and responds with good status the user callback function
*              is called. If bad status is returned the thread releases
*              control of the interrupt level/vector and exits.
*
*  PARAMETERS: fdesc (I): File descriptor from open
*              VmeLevel (I): VME interrupt to use (1 - 7) 
*              VmeVector (I): VME acknowledge vector to use 
*              * InterStatus (I/O): Interrupt status
*
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*              UVI_BAD_PARAMETER_1 if level is out of range
*              UVI_BAD_PARAMETER_2 if vector is out of range
*              UVI_INTR_IN_USE if level/vector already in use
*
*  GLOBALS:    None
*
******************************************************************************/

int 
VmeWaitInt( UVI_UINT32 fdesc, UVI_UINT8 VmeLevel, UVI_UINT8 VmeVector, UVI_UINT32 * InterStatus ) 
{
    int ret_status = 0;
    struct uviInterruptParameters IntParams;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }

    IntParams.Fdesc = fdesc;
    IntParams.IntLevel = VmeLevel;
    IntParams.IntVector = VmeVector;
    IntParams.IntStatus = 0;

    /* The IOCTL process sleeps waiting interrupt */
    ret_status = ioctl( fdesc, IOCTL_UVI_WAIT_VME_INT, &IntParams );

    *InterStatus = IntParams.IntStatus;

    return( ret_status );

}  /* end of VmeWaitInt() */


/******************************************************************************
*
*  FUNCTION:   RequestVmeMB 
*
*  PURPOSE:    Request VME mailbox
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              MailboxNum (I): One of four mailboxes 
*              
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*  GLOBALS:    None
*
******************************************************************************/

int 
RequestVmeMB( UVI_UINT32 fdesc, UVI_UINT32 MailboxNum )
{
    UVI_UINT32 mbnum;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }
        
    mbnum = MailboxNum;

    if ( ioctl(fdesc, IOCTL_UVI_REQ_MB, &mbnum ) != 0 )
    {
	return( UVI_OS_ERROR );
    }

    return( UVI_SUCCESS );

} /* end RequestVmeMB */


/******************************************************************************
*
*  FUNCTION:   ReleaseVmeMB 
*
*  PURPOSE:    Release VME mailbox
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              MailboxNum (I): One of four mailboxes 
*              
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*  GLOBALS:    None
*
******************************************************************************/

int 
ReleaseVmeMB( UVI_UINT32 fdesc, UVI_UINT32 MailboxNum )
{
    UVI_UINT32 mbnum;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }
        
    mbnum = MailboxNum;

    if ( ioctl(fdesc, IOCTL_UVI_RELEASE_MB, &mbnum ) != 0 )
    {
	return( UVI_OS_ERROR );
    }

    return( UVI_SUCCESS );

} /* end ReleaseVmeMB */


/******************************************************************************
*
*  FUNCTION:   CancelWaitVmeMB 
*
*  PURPOSE:    Cancel wait for VME mailbox interrupt
*  PARAMETERS: fdesc (I): File descriptor for the Universe driver
*              MailboxNum (I): One of four mailboxes 
*              
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*  GLOBALS:    None
*
******************************************************************************/

int 
CancelWaitVmeMB( UVI_UINT32 fdesc, UVI_UINT32 MailboxNum )
{
    UVI_UINT32 mbnum;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }
        
    mbnum = MailboxNum;

    if ( ioctl(fdesc, IOCTL_UVI_CANCEL_MB, &mbnum ) != 0 )
    {
	return( UVI_OS_ERROR );
    }

    return( UVI_SUCCESS );

} /* end CancelWaitVmeMB */


/******************************************************************************
*
*  FUNCTION:   WaitVmeMBInt
*
*  PURPOSE:    Wait for VME mailbox interrupt
*              Mailbox resource must already be locked by user.
*
*  PARAMETERS: fdesc (I): File descriptor from open
*              VmeMailbox (I): VME mailbox to use (0 -3) 
*              MBdat (I/O): Function parameter for mailbox data
*  RETURNS:    UVI_SUCCESS if no errors occur
*              UVI_NO_UDEV if the driver's file descriptor is invalid
*              UVI_OS_ERROR if the ioctl(2) fails
*  GLOBALS:    None
*
******************************************************************************/

int 
WaitVmeMBInt( UVI_UINT32 fdesc, UVI_UINT32 VmeMailbox, UVI_UINT32 * MBdat )
{
    int ret_status = 0;
    UVI_UINT32 mbnum;
    struct uviMailboxParameters MBparams;

    mbnum = VmeMailbox;

    /* Check for valid file descriptor */
    if( fdesc < 0 )
    {
        return( UVI_NO_UDEV );
    }

    MBparams.Fdesc = fdesc;
    MBparams.MailboxNum = VmeMailbox;
    MBparams.MailboxDat = 0;

    /* The IOCTL process sleeps waiting interrupt */
    ret_status = ioctl( fdesc, IOCTL_UVI_ENABLE_MB, &MBparams );

    *MBdat = MBparams.MailboxDat;

    return( ret_status );

}  /* end of VmeMB() */



