
#include "vmic_common_test.c"


/*
 * Handler for Ctrl-C
 */
static void ctlc_handler( int v );


/*
 * These variables must be global to be accessed by the Ctrl-C handler
 */
int fd, level, vector;


/*
 * Set up an interrupt handler to catch mailbox interrupts.
 *
 * USAGE:
 * catch_int level vector
 *
 * level - VMEbus interrupt level
 *
 * vector - VMEbus interrupt vector
 *
 * RETURNS:
 *   0 on success or -1 on error.
 */
int main( int argc, char **argv )
{
  UVI_UINT32        intstat;
  struct sigaction  act;

  if( argc < 3 )
    {
      fprintf( stderr, "vmb level vector\n" );
      return( -1 );
    }

  if( ( level = strtol( argv[1], NULL, 0 ) ) < 0 )
    {
      fprintf( stderr, "Invalid level argument\n" );
      return( -1 );
    }

  if( ( vector = strtol( argv[2], NULL, 0 ) ) < 0 )
    {
      fprintf( stderr, "Invalid vector argument\n" );
      return( -1 );
    }

  /*
   * We do not want to mask any interrupts while handling this signal
   */
  sigemptyset( &act.sa_mask );
  act.sa_handler = ctlc_handler;
  act.sa_flags = 0;
  
  /*
   * Set the interrupt handler for SIGKILL
   */
  if( sigaction( SIGINT, &act, NULL ) < 0 )
    {
      perror( "sigaction" );
      return( -1 );
    }

  if( ( fd = VmeOpen( ) ) < 0 )
    {
      perror( "VmeOpen" );
      return( -1 );
    }

  if( RequestVmeInt( fd, level, vector ) != UVI_SUCCESS )
    {
      perror( "RequestVmeInt" );
      VmeClose( fd );
      return( -1 );
    }

  if( VmeWaitInt( fd, level, vector, &intstat ) != UVI_SUCCESS )
    {
      perror( "VmeWaitInt" );
      ReleaseVmeInt( fd, level, vector );
      VmeClose( fd );
      return( -1 );
    }

  DPRINTF(( stderr, "interrupt status %x\n", intstat ));

  /*
   * Test for IACK Bus error, else print level and vector
   */
  if( intstat & 0x10000 )
    fprintf( stderr, "Received IACK bus error\n" );
  else
    fprintf( stdout, "Level %x, Vector %x\n", ( intstat & 0xff00 ) >> 8,
	     intstat & 0xff );

  if( ReleaseVmeInt( fd, level, vector ) != UVI_SUCCESS )
    {
      perror( "ReleaseVmeInt" );
      VmeClose( fd );
      return( -1 );
    }

  if( VmeClose( fd )!= UVI_SUCCESS )
    {
      perror( "VmeClose" );
      return( -1 );
    }

  return( 0 );
}


static void ctlc_handler( int v )
{

  DPRINTF(( stderr, "Termination signal received\n" ));
  ReleaseVmeInt( fd, level, vector );
  VmeClose( fd );
  exit( -1 );
}
