
#include "vmic_common_test.c"


/*
 * Read a value from some VMEBus address.
 *
 * USAGE:
 * dread [-a address modifier] [-d data width] address [length]
 *
 * address modifier - VMEbus address modifier [A16UP, A16UD, A16SP, A16SD,
 *                                             A24UP, A24UD, A24SP, A24SD,
 *                                             A32UP, A32UD, A32SP, A32SD ]
 *
 * data width - VMEbus access data width [8, 16, 32]
 *
 * address - VMEbus address
 *
 * length - Optional number of elements to print. Default is 1.
 *
 * RETURNS:
 *   0 on success or -1 on error.
 */
int main( int argc, char **argv )
{
  int            c, fd;
  unsigned       size, buffsize;
  char           *buffer;
  UVI_UINT32     vaddr, addr_mod, dwidth, round, status;

  addr_mod = DEFAULT_ADDR_MOD;
  dwidth = DEFAULT_DWIDTH;

  while( ( c = getopt( argc, argv, "a:d:" ) ) != -1 )
    switch( c )
      {
      case 'a' :
	if( ( addr_mod = strtoam( optarg ) ) < 0 )
	  {
	    fprintf( stderr, "Invalid address modifier\n" );
	    return( -1 );
	  }
	break;
      case 'd' :
	if( ( dwidth = strtodw( optarg ) ) < 0 )
	  {
	    fprintf( stderr, "Invalid data width\n" );
	    return( -1 );
	  }
	break;
      case '?' :
      default :
	fprintf( stderr, "dread [-a address modifier] [-d data width] "
		 "address [length]\n" );
	return( -1 );
      }

  if( argv[optind] )
    vaddr = strtoul( argv[optind++], NULL, 0 );
  else
    {
      fprintf( stderr, "No address specified\n" );
      return( -1 );
    }

  size = ( argc > optind ) ? strtol( argv[optind], NULL, 0 ) : 1;
  size *= ( dwidth / 8 );

  DPRINTF(( stderr, "reading %d bytes, address 0x%x, modifier 0x%x, "
	    "data width %d bits\n", size, vaddr, addr_mod, dwidth ));

  if( ( fd = VmeOpen( ) ) < 0 )
    {
      perror( "VmeOpen" );
      return( -1 );
    }

  /*
   * The access must be rounded to a data width boundary
   */
  round = ( vaddr & ( PAGESIZE - 1 ) ) % dwidth;

  buffsize = ( PAGESIZE + size )  & ~( PAGESIZE - 1 );

  DPRINTF(( stdout, "mapping 0x%x bytes of memory for DMA at 0x%x\n",
	    buffsize, DMA_BUFF_BASE + round ));

  /*
   * Map the buffer to the memory segment reserved at boot-up
   */
  if( MAP_FAILED == ( buffer = mmap( 0, buffsize,
				     PROT_READ | PROT_WRITE,
				     MAP_SHARED, fd, DMA_BUFF_BASE ) ) )
    {
      perror( "mmap" );
      VmeClose( fd );
      return( -1 );
    }

  /*
   * Request the DMA resource
   */
  if ( VmeReqDma( fd ) < 0 )
    {
      perror( "VmeReqDma" );
      munmap( buffer, buffsize );
      VmeClose( fd );
      return( -1 );
    }

  status = 0;

  if( VmeDmaIO( fd, vaddr - round, addr_mod, size + round, dwidth,
		DMA_RD, 0, &status ) != UVI_SUCCESS )
    {
      perror( "VmeDmaIO" );
      VmeReleaseDma( fd );
      munmap( buffer, buffsize );
      VmeClose( fd );
      return( -1 );
    }

  /*
   * Release the DMA resource
   */
  VmeReleaseDma( fd );

  if( status & 0x700 )
    {
      fprintf( stderr, "DMA returned error\n" );
      munmap( buffer, buffsize );
      VmeClose( fd );
      return( -1 );
    }

  if( dump( buffer + round, size, dwidth ) < 0 )
    {
      perror( "dump" );
      munmap( buffer, buffsize );
      VmeClose( fd );
      return( -1 );
    }

  if( munmap( buffer, buffsize ) < 0 )
    {
      perror( "munmap" );
      VmeClose( fd );
      return( -1 );
    }

  if( VmeClose( fd )!= UVI_SUCCESS )
    {
      perror( "VmeClose" );
      return( -1 );
    }

  return( 0 );
}
