
#include "vmic_common_test.c"


/*
 * Write a value from some VMEBus address using DMA.
 *
 * USAGE:
 * dwrite [-a address modifier] [-d data width] address hexvalue ...
 *
 * address modifier - VMEbus address modifier [A16UP, A16UD, A16SP, A16SD,
 *                                             A24UP, A24UD, A24SP, A24SD,
 *                                             A32UP, A32UD, A32SP, A32SD ]
 *
 * data width - VMEbus access data width [8, 16, 32]
 *
 * address - VMEbus address
 *
 * hexvalue - Hexidecimal byte value to be set at that offset.  Value may be \
 *            a list of values to be set at successive byte locations from \
 *            offset.
 *
 * RETURNS:
 *   0 on success or -1 on error.
 */
int main( int argc, char **argv )
{
  void           *data;
  int            c, fd;
  unsigned       size, buffsize;
  char           *buffer;
  UVIHANDLE      handle;
  UVI_UINT32     vaddr, addr_mod, dwidth, status;

  addr_mod = DEFAULT_ADDR_MOD;
  dwidth = DEFAULT_DWIDTH;

  while( ( c = getopt( argc, argv, "a:d:" ) ) != -1 )
    switch( c )
      {
      case 'a' :
	if( ( addr_mod = strtoam( optarg ) ) < 0 )
	  {
	    fprintf( stderr, "Invalid address modifier\n" );
	    return( 1 );
	  }
	break;
      case 'd' :
	if( ( dwidth = strtodw( optarg ) ) < 0 )
	  {
	    fprintf( stderr, "Invalid data width\n" );
	    return( -1 );
	  }
	break;
      case '?' :
      default :
	fprintf( stderr, "dwrite [-a address modifier] [-d data width] "
		 "address hexvalue ...\n" );
	return( -1 );
      }

  if( argv[optind] )
    vaddr = strtoul( argv[optind++], NULL, 0 );
  else
    {
      fprintf( stderr, "No address specified\n" );
      return( -1 );
    }

  /*
   * Test the address to see if it is properly aligned
   */
  if( vaddr % dwidth )
    {
      fprintf( stderr, "The VMEbus address must be data width aligned\n" );
      return( -1 );
    }

  /*
   * Get data from argument list
   */
  size = ( argc - optind ) * ( dwidth / 8 );
  if( convert_hexdata( &data, size, dwidth, argv + optind ) < 0 )
    {
      perror( "convert_hexdata" );
      return( -1 );
    }

  DPRINTF(( stdout, "Writing %d bytes, address 0x%x, modifier 0x%x, "
	    "data width %d bits\n", size, vaddr, addr_mod, dwidth ));

  if( ( fd = VmeOpen( ) ) < 0 )
    {
      perror( "VmeOpen" );
      return( -1 );
    }

  buffsize = ( PAGESIZE + size )  & ~( PAGESIZE - 1 );

  DPRINTF(( stdout, "mapping 0x%x bytes of memory for DMA at 0x%x\n",
	    buffsize, DMA_BUFF_BASE ));

  /*
   * Map the buffer to the memory segment reserved at boot-up
   */
  if( MAP_FAILED == ( buffer = mmap( 0, buffsize,
				     PROT_READ | PROT_WRITE,
				     MAP_SHARED, fd, DMA_BUFF_BASE ) ) )
    {
      perror( "mmap" );
      VmeClose( fd );
      return( -1 );
    }

  memcpy( buffer, data, size );

  /*
   * Request the DMA resource
   */
  if ( VmeReqDma( fd ) < 0 )
    {
      perror( "VmeReqDma" );
      munmap( buffer, buffsize );
      VmeClose( fd );
      return( -1 );
    }

  status = 0;

  if( VmeDmaIO( fd, vaddr, addr_mod, size, dwidth, DMA_WR, 0, &status )
      != UVI_SUCCESS )
    {
      perror( "VmeDmaIO" );
      VmeReleaseDma( fd );
      munmap( buffer, buffsize );
      VmeClose( fd );
      return( -1 );
    }

  /*
   * Release the DMA resource
   */
  VmeReleaseDma( fd );

  if( status & 0x700 )
    {
      fprintf( stderr, "DMA returned error\n" );
      munmap( buffer, buffsize );
      VmeClose( fd );
      return( -1 );
    }

  if( munmap( buffer, buffsize ) < 0 )
    {
      perror( "munmap" );
      VmeClose( fd );
      return( -1 );
    }

  if( VmeClose( fd )!= UVI_SUCCESS )
    {
      perror( "VmeClose" );
      return( -1 );
    }

  return( 0 );
}
