#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>
#include <uvimodule/uvi_api.h>
#include <uvimodule/uvi_universe.h>


#define DEFAULT_ADDR_MOD V_A32SD
#define DEFAULT_DWIDTH   32
#define DEFAULT_VAS      VSI_CTL_A32

#define PAGESIZE 0x1000

#ifdef DEBUG
#define DPRINTF( x )  fprintf x
#else
#define DPRINTF( x )
#endif


/*
 * Convert an address modifier string to it's integer value
 */
static int strtoam( char *str )
{
  char *table[0x40] =
  { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL, "A32UD", "A32UP", "A32BT", NULL, "A32SD", "A32SP", "A32BT",
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL, "A16UD", NULL, NULL, NULL, "A16SD", NULL, NULL,
    NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    NULL, "A24UD", "A24UP", "A24BT", NULL, "A24SD", "A24SP", "A24BT" };
  int ii;

  for( ii = 0; ii < 0x40; ii++ )
    if( table[ii] )
      if( 0 == strcmp( str, table[ii] ) )
	return( ii );

  return( -1 );
}


/*
 * Convert a data width string argument to a data width value
 */
static int strtodw( char *str )
{
  int dwidth;

  if( ( dwidth = strtol( optarg, NULL, 0 ) ) < 0 )
    {
      fprintf( stderr, "Invalid data width\n" );
      return( -1 );
    }

  /*
   * Test for validity
   */
  switch( dwidth )
    {
    case UVI_BYTE :
    case UVI_WORD :
    case UVI_LWORD :
      return( dwidth );
      break;
    default :
      return( -1 );
      break;
    }
}


static int dump( void *data, int nbytes, int width )
{
#define LF   0x10
  int ii;

  switch( width )
    {
    case UVI_BYTE :
      {
	unsigned char *w = data;
	for( ii = 0; ii < nbytes; w++ )
	  printf( "%.2x%c", *w, ( ++ii % LF ) ? ' ' : '\n' );
      }
      break;
    case UVI_WORD :
      {
	unsigned short *w = data;
	for( ii = 0; ii < ( nbytes / 2 ); w++ )
	  printf( "%.4x%c", *w, ( ( ++ii * 2 ) % LF ) ? ' ' : '\n' );
      }
      break;
    case UVI_LWORD :
      {
	unsigned long *w = data;
	for( ii = 0; ii < ( nbytes / 4 ); w++ )
	  printf( "%.8x%c", *w, ( ( ++ii * 4 ) % LF ) ? ' ' : '\n' );
      }
      break;
    default :
      errno = EINVAL;
      return( -1 );
    }

  printf( "\n" );
  return( 0 );
}


/*
 * copy data using the width specified
 */
static int vmemcpy( void *dest, const void *src, int nbytes, int width )
{
  int ii;

  switch( width )
    {
    case UVI_BYTE :
      {
	const unsigned char *s = src;
	unsigned char *d = dest;
	for( ii = 0; ii < nbytes; ++ii, ++s, ++d )
	    *d = *s;
      }
      break;
    case UVI_WORD :
      {
	const unsigned short *s = src;
	unsigned short *d = dest;
	for( ii = 0; ii < ( nbytes / 2 ); ++ii, ++s, ++d )
	  *d = *s;
      }
      break;
    case UVI_LWORD :
      {
	const unsigned long *s = src;
	unsigned long *d = dest;
	for( ii = 0; ii < ( nbytes / 4 ); ++ii, ++s, ++d )
	  *d = *s;
      }
      break;
    default :
      errno = EINVAL;
      return( -1 );
    }

  return( 0 );
}


/*
 * Get hex arguments and convert them to an array of hex values
 */
static int convert_hexdata( void **data, int nbytes, int width, char **argv )
{
  int ii;

  if( ( *data = ( void * )malloc( nbytes ) ) == NULL )
    {
      perror( "malloc" );
      return( -1 );
    }

  switch ( width )
    {
    case UVI_BYTE :
      {
	unsigned char *d = *data;
	for( ii = 0; ii < nbytes; ++ii, ++d )
	  *d = strtoul( argv[ii], NULL, 16 );
      }
      break;
    case UVI_WORD :
      {
	unsigned short *d = *data;
	for( ii = 0; ii < ( nbytes / 2 ); ++ii, ++d )
	  *d = strtoul( argv[ii], NULL, 16 );
      }
      break;
    case UVI_LWORD :
      {
	unsigned long *d = *data;
	for( ii = 0; ii < ( nbytes / 4 ); ++ii, ++d )
	  *d = strtoul( argv[ii], NULL, 16 );
      }
      break;
    default :
      free( *data );
      errno = EINVAL;
      return( -1 );
      break;
    }

  return( 0 );
}
