
#include "vmic_common_test.c"


/*
 * Create a VMEbus slave window
 *
 * USAGE:
 * vslave [-a address space] [-o offset] [-w window] address size
 *
 * address space - VMEbus address space [A16, A24, A32]
 *
 * offset - offset from SLAVE_BUFF_BASE
 *
 * window - slave window number
 *
 * address - VMEbus address of the slave window
 *
 * length - Number of bytes to slave out
 *
 * RETURNS:
 *   0 on success or -1 on error.
 */
int main( int argc, char **argv )
{
  int         c, fd;
  unsigned    size, buffsize;
  UVI_UINT32  vaddr, vas, offset, window;

  offset = 0;
  window = 0;
  vas = DEFAULT_VAS;

  while( ( c = getopt( argc, argv, "a:o:w:" ) ) != -1 )
    switch( c )
      {
      case 'a' :
	if( 0 == strcmp( optarg, "A16" ) )
	  vas = VSI_CTL_A16;
	else if( 0 == strcmp( optarg, "A24" ) )
	  vas = VSI_CTL_A24;
	else if( 0 == strcmp( optarg, "A32" ) )
	  vas = VSI_CTL_A32;
	else
	  {
	    fprintf( stderr, "Invalid address space argument\n" );
	    return( -1 );
	  }
	break;
      case 'o' :
	if ( ( offset = strtoul( optarg, NULL, 0 ) ) < 0 )
	  {
	    fprintf( stderr, "Invalid offset\n" );
	    return( -1 );
	  }
	break;
      case 'w' :
	if ( ( window = strtoul( optarg, NULL, 0 ) ) < 0 )
	  {
	    fprintf( stderr, "Invalid offset\n" );
	    return( -1 );
	  }
	break;
      case '?' :
      default :
	fprintf( stderr, "vslave [-a address space] [-o offset] [-w window] "
		 "[-d data width] size\n" );
		 
	return( -1 );
      }

  if( argv[optind] )
    vaddr = strtoul( argv[optind++], NULL, 0 );
  else
    {
      fprintf( stderr, "No address specified\n" );
      return( -1 );
    }

  if( argv[optind] )
    size = strtol( argv[optind], NULL, 0 );
  else
    {
      fprintf( stderr, "Size not specified\n" );
      return( -1 );
    }

  /*
   * Validate the window and it's parameters
   */
  switch( window )
    {
    case 0 :
    case 4 :
      if( ( size % 0x1000 ) || ( vaddr % 0x1000 ) )
	{
	  fprintf( stderr, "Windows 0 and 4 must have 4kb resolution\n" );
	  return( -1 );
	}
      break;
    case 1 :
    case 2 :
    case 3 :
    case 5 :
    case 6 :
    case 7 :
      if( ( size % 0x10000 ) || ( vaddr % 0x10000 ) )
	{
	  fprintf( stderr, "Windows 1-3, 5-7 must have 64kb resolution\n" ); 
	  return( -1 );
	}
      break;
    default :
      fprintf( stderr, "Invalid window\n" );
      return( -1 );
      break;
    }

  if( ( fd = VmeOpen( ) ) < 0 )
    {
      perror( "VmeOpen" );
      return( -1 );
    }

  DPRINTF(( stderr, "Opening slave window %d,  0x%x bytes, "
	    "VMEbus address 0x%x, PCI address 0x%x, VAS 0x%x\n",
	    size, window, vaddr, SLAVE_BUFF_BASE + offset, vas ));

  /*
   * For simplicity we set the slave ram to be both superviser/user accessible
   * and program/data accessible
   */
  if( OpenVmeSlave( fd, vaddr, VSI_CTL_SU, VSI_CTL_PD, vas,
		    size, offset, window ) != UVI_SUCCESS )
    {
      perror( "OpenVmeSlave" );
      VmeClose( fd );
      return( -1 );
    }

  fprintf( stderr,
	   "Press enter to close the slave window\n" );
  getchar( );

  if( CloseVmeSlave( fd, window ) != UVI_SUCCESS )
    {
      perror( "CloseVmeSlave" );
      VmeClose( fd );
      return( -1 );
    }

  if( VmeClose( fd )!= UVI_SUCCESS )
    {
      perror( "VmeClose" );
      return( -1 );
    }

  return( 0 );
}
