/*
=================================================================================
                            COPYRIGHT NOTICE

    Copyright (C) 2000 VMIC
    International Copyright Secured.  All Rights Reserved.

---------------------------------------------------------------------------------
    Filename: uvidev_init.c  Version: 1.1  Modification Date: 00/03/21 10:15:02
---------------------------------------------------------------------------------
    Description: Initialize the Universe device.
---------------------------------------------------------------------------------

    Revision History:
    Date         Who  Ver   Reason For Change
    -----------  ---  ----  -----------------------------------------------------
    21-MAR-2000  AE   1.1   Created.

=================================================================================
*/
#ident "@(#)uvidev_init.c 1.1 00/06/27 VMIC"


#include "uvi_module.h"

/********************************************************************************
*
*  FUNCTION:   uviInitDev
*
*  PURPOSE:    Check PCI regions for possible memory conflicts.
*              Find & initialize the PLX device if one exists for the CPU being
*              used.  If no PLX, try to initialize the COMM command register.  
*              Also find & initialize the universe device.  This initialization
*              does not include items that an external kernel module or a user 
*              application will be responsible for requesting.
*
*  PARAMETERS: void
*
*  RETURNS:    UVI_SUCCESS       : Success
*              UVI_NODEV_FOUND   : No PLX device or ISA/COMM registers found
*              UVI_CPU_NOTREC    : Cpu not recognized as one of VMIC's
*              UVI_NULL_PTR      : Null pointer returned
*              UVI_NOT_SUPPORTED : Operation/mode not presently supported
*              UVI_MAP_ERROR     : Mapping operation failed
*              UVI_DRIVER_ERROR  : driver operation/function failed
*
*  GLOBALS:    UdeviceOk, UnivBaseAddr, PlxMappedAddr, pci_devices, 
*              AllPciDevices, PlxFound, UviPlxDevice, IsaIdRegVal, UnivFound, 
*              UviDevice, Univ1Found, UnivRevId, UnivIrq
*
********************************************************************************/
UVI_INT32 uviInitDev( void )
{
    static char *ThisFunction = "uviInitDev";

    UVI_UINT16 NumVmeWindows,
               IsaCommandBits;

    UVI_UINT32 PciIoAddr,
               UnivPciId;
               
    UVI_INT32  GetPciRegionsStatus,
               uviInitVmeWindStatus,
               uviInitMailboxStatus;

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Entering ...\n", MODULE_NAME, ThisFunction );

    UdeviceOk = UVI_FALSE;
    UnivBaseAddr = 0;
    PlxMappedAddr = 0;

/*
** use linux ptr to system list of PCI devices & check for good list.
*/
    if ( pci_devices != (struct pci_dev *)NULL )
    {

/*
** Let's find all PCI regions to avoid mapping problems later.  If we have a
** memory area conflict, this may or may not be a 'show stopper'.  How to handle?
*/
        GetPciRegionsStatus = uviGetPciRegions();
        if ( GetPciRegionsStatus != UVI_SUCCESS )
        {
            WHENDEBUG( UVI_DBEXTRA )
            {
                printk( KERN_ERR "%s:%s; Memory that is needed for VME windows is already"
                        " in use!\n", MODULE_NAME, ThisFunction );
            }
        }

/* Start the search for the Universe/PLX devices. */
        WHENDEBUG( UVI_DBINIT )
        {
            printk( KERN_INFO "%s:%s; Trying to find the Universe/PLX/ISA interfaces.\n",
                    MODULE_NAME, ThisFunction );
        }

/*
** Search for the PLX device first.  If not found, set up to access COMM reg
** D800E + 8.  Look for 7XXX identifier to test for VMIC cpu boards.  Access
** the PLX style regs using the COMM reg base address.
*/
        AllPciDevices = pci_devices;
        PlxFound = UVI_FALSE;
        do
        {
            if ( ( AllPciDevices->vendor == PLX_VID ) &&
                                            ( AllPciDevices->device == PLX_DID ) )
            {
                PlxFound = UVI_TRUE;
                UviPlxDevice = AllPciDevices;
            }
            else
            {
                AllPciDevices = AllPciDevices->next;
            }
        } while ( ( !PlxFound ) && ( AllPciDevices != ( struct pci_dev * )NULL) );

        if ( !PlxFound )
        {
/* Try using the COMM reg. */
            IsaIdRegVal = readw( NOTPLX_BRDID_ADDR );
            if ( IsaIdRegVal == (UVI_UINT16)NULL )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; No board ID returned from ISA access,"
                            " this is a problem!\n", MODULE_NAME, ThisFunction );
                }
                return( UVI_NODEV_FOUND );
            }
/*            WHENDEBUG( UVI_DBEXTRA )
            {
                printk( KERN_INFO "%s:%s; Vmic cpu board id reg val is 0x%04x.\n",
                        MODULE_NAME, ThisFunction, IsaIdRegVal );
            }
*/
            if ( ( IsaIdRegVal >= 0x7591 ) && ( IsaIdRegVal <= 0x7999 ) )
            {
                WHENDEBUG( UVI_DBINIT )
                {
                    printk( KERN_INFO "%s:%s; Found vmic cpu class 0x%04x.\n", MODULE_NAME,
                            ThisFunction, IsaIdRegVal );
                }
            }
            else
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; CPU board not recognized, may not"
                            " be a VMIC CPU!\n", MODULE_NAME, ThisFunction );
                }
                return( UVI_CPU_NOTREC );
            }
        }

/*
** Search for the universe device.  If found, determine whether we have a universe
** I or a universe II so that we'll know how many vme windows are supported.
*/
        AllPciDevices = pci_devices;
        UnivFound = UVI_FALSE;
        do
        {
            if ( ( AllPciDevices->vendor == UNIV_VID ) &&
                                            ( AllPciDevices->device == UNIV_DID ) )
            {
                UnivFound = UVI_TRUE;
                UviDevice = AllPciDevices;
            }
            else
            {
                AllPciDevices = AllPciDevices->next;
            }
        } while ( ( !UnivFound ) && ( AllPciDevices != ( struct pci_dev * )NULL) );

        if ( UnivFound )
        {
/* Map the universe control register base address. */
            PciIoAddr = UviDevice->base_address[0];
            if ( ( PciIoAddr & PCI_BASE_ADDRESS_SPACE )
                                              == PCI_BASE_ADDRESS_SPACE_MEMORY )
            {
                WHENDEBUG( UVI_DBINIT )
                {
                    printk( KERN_INFO "%s:%s; Universe device uses memory address 0x%lx.\n",
                            MODULE_NAME, ThisFunction, PciIoAddr & PCI_BASE_ADDRESS_MEM_MASK );
                }
                UnivBaseAddr = (char *)ioremap_nocache( PciIoAddr &
                                           PCI_BASE_ADDRESS_MEM_MASK, 0x0FFF );
                if ( UnivBaseAddr == NULL )
                {
                    WHENDEBUG( UVI_DBERROR )
                    {
                    printk( KERN_ERR "%s:%s; NULL returned from UCSR ioremap, this"
                            " is a problem!\n", MODULE_NAME, ThisFunction );
                    }
                    return( UVI_NULL_PTR );
                }
            }
            else
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; Universe device not in memory space!\n",
                            MODULE_NAME, ThisFunction );
                }
                return( UVI_NOT_SUPPORTED );
            }

/* Check to see if the mapping worked out. Put this section in a 'WHENDEBUG' macro. */
            WHENDEBUG( UVI_DBEXTRA )
            {
                UnivPciId = *(UVI_UINT32 *)( UnivBaseAddr + PCI_ID );
                if ( UnivPciId == (UVI_UINT32)NULL )
                {
                    printk( KERN_ERR "%s:%s; NULL returned from UCSR read,"
                                " this is a problem!\n", MODULE_NAME, ThisFunction );
                    return( UVI_NULL_PTR );
                }
                printk( KERN_INFO "%s:%s; Read via mapping, PCI_ID = 0x%08X\n",
                        MODULE_NAME, ThisFunction, UnivPciId );
                if ( UnivPciId != UNIV_VID )
                {
                    printk( KERN_INFO "%s:%s; Universe chip failed to return"
                            " PCI_ID in memory map.\n", MODULE_NAME, ThisFunction );
                    return( UVI_MAP_ERROR );
                }
            }

/*
** Check to see that PCI bus mastering is enabled for the Universe device.  If not,
** DMA, and VME slave windows, may not work.
*/
            if ( ( *(UVI_UINT32 *)( UnivBaseAddr + PCI_CSR ) && 0x4 ) == UVI_FALSE )
            {
                WHENDEBUG( UVI_DBINIT )
                {
                    printk( KERN_INFO "%s:%s; Setting universe device as pci bus mstr.\n",
                            MODULE_NAME, ThisFunction );
                }
                pci_set_master( UviDevice );
            }           

/*
** Determine whether we have a universe 1 or a universe 2 type device.  We do this
** by checking the PCI_CLASS reg.
*/
            Univ1Found = UVI_FALSE;

            UnivRevId = *( UnivBaseAddr + PCI_CLASS ) & 0x1; /* get the rev id */
            if ( UnivRevId == 0 ) /* 0 for universe I, 1 for universe II */
            {
                WHENDEBUG( UVI_DBINIT )
                {
                    printk( KERN_INFO "%s:%s; Universe interface is type 1.\n",
                            MODULE_NAME, ThisFunction );
                }
                Univ1Found = UVI_TRUE;
                NumVmeWindows = 4;
            }
            else
            {
                WHENDEBUG( UVI_DBINIT )
                {
                    printk( KERN_INFO "%s:%s; Universe interface is type 2.\n",
                            MODULE_NAME, ThisFunction );
                }
                NumVmeWindows = 8;
            }
        }
        else
        {
            WHENDEBUG( UVI_DBERROR )
            {
                printk( KERN_ERR "%s:%s; No universe interface found, abort!\n",
                MODULE_NAME, ThisFunction );
            }
            return( UVI_NODEV_FOUND );
        }

/*
** Everything is ok so lets set up the univ master control reg.  If universe type
** is 1, then write 0 to MAXRTRY due to bug mentioned in universe docs.
*/
        if ( Univ1Found )
        {
            *(UVI_UINT32 *)( UnivBaseAddr + MAST_CTL ) = MAXRTRY( 0 ) |
                                                         PWON_CNT_128 |
                                                         VRL_3 |
                                                         VRM_DEMAND |
                                                         VREL_WHEN_DONE |
                                                         PABS_32 |
                                                         BUS_NO( 0 );
        }
        else
        {
            *(UVI_UINT32 *)( UnivBaseAddr + MAST_CTL ) = MAXRTRY( 512 ) |
                                                         PWON_CNT_128 |
                                                         VRL_3 |
                                                         VRM_DEMAND |
                                                         VREL_WHEN_DONE |
                                                         PABS_32 |
                                                         BUS_NO( 0 );
        }

/* Set up the universe misc control reg.  If autodetect senses system controller ... */
        if ( *(UVI_UINT32 *)( UnivBaseAddr + MISC_CTRL ) & SYSCON )
        {
            *(UVI_UINT32 *)( UnivBaseAddr + MISC_CTRL ) = VBTO_64US |
                                                          VARB_PRIORITY |
                                                          VARB_TOUT_16US |
                                                          RESCIND_DTACK_EN |
                                                          SYSCON;
        }
/* ... not system controller */
        else
        {
            *(UVI_UINT32 *)( UnivBaseAddr + MISC_CTRL ) = ( VBTO_64US |
                                            VARB_PRIORITY |
                                            VARB_TOUT_16US |
                                            RESCIND_DTACK_EN ) & ( ~SYSCON );
        }

/* Set up the PCI target windows. */
        uviInitVmeWindStatus = uviInitVmeWindows();
        if ( uviInitVmeWindStatus != UVI_SUCCESS )
        {
            WHENDEBUG( UVI_DBERROR )
            {
                printk( KERN_ERR "%s:%s; Failed to initialize the vme windows,"
                        " abort!\n", MODULE_NAME, ThisFunction );
            }
            return( UVI_DRIVER_ERROR );
        }

/* Set up the Mail Box access. */ 
        uviInitMailboxStatus = uviInitMailboxes(); 
        if ( uviInitMailboxStatus != UVI_SUCCESS ) 
        { 
            WHENDEBUG( UVI_DBERROR ) 
            { 
                printk( KERN_ERR "%s:%s; Failed to initialize the Mailboxes," 
                        " abort!\n", MODULE_NAME, ThisFunction ); 
            } 
            return( UVI_DRIVER_ERROR ); 
        } 
 
/*
** Set up the VME bus interrupts.  Get the universe IRQ value and disable interrupt
** registers to start with.
*/
        UnivIrq = UviDevice->irq;
        WHENDEBUG( UVI_DBEXTRA )
        {
            printk( KERN_INFO "%s:%s; The universe device IRQ value is 0x%x.\n",
                    MODULE_NAME, ThisFunction, UnivIrq );
        }

        if ( Univ1Found )
        {
            *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= LINT_DIS_ALL_U1;
            *(UVI_UINT32 *)( UnivBaseAddr + VINT_EN ) &= VINT_DIS_ALL_U1;
        }
        else
        {
            *(UVI_UINT32 *)( UnivBaseAddr + LINT_EN ) &= LINT_DIS_ALL_U2;
            *(UVI_UINT32 *)( UnivBaseAddr + VINT_EN ) &= VINT_DIS_ALL_U2;
        }

/* Map all interrupts to INTA - register bits same for U1 and U2 */
        *(UVI_UINT32 *)( UnivBaseAddr + LINT_MAP0 ) = VIRQ7_TO_PCI( 0 ) |
                                                      VIRQ6_TO_PCI( 0 ) |
                                                      VIRQ5_TO_PCI( 0 ) |
                                                      VIRQ4_TO_PCI( 0 ) |
                                                      VIRQ3_TO_PCI( 0 ) |
                                                      VIRQ2_TO_PCI( 0 ) |
                                                      VIRQ1_TO_PCI( 0 ) |
                                                      VOWN_TO_PCI( 0 );

        *(UVI_UINT32 *)( UnivBaseAddr + LINT_MAP1 ) = ACFAIL_TO_PCI( 0 ) |
                                                      SYSFAIL_TO_PCI( 0 ) |
                                                      SW_INT_TO_PCI( 0 ) |
                                                      SW_IACK_TO_PCI( 0 ) |
                                                      VERR_TO_PCI( 0 ) |
                                                      LERR_TO_PCI( 0 ) |
                                                      DMA_TO_PCI( 0 );

/* Enable the vme bus now. */
        WHENDEBUG( UVI_DBINIT )
        {
            printk( KERN_INFO "%s:%s; Enabling the vme bus.\n", MODULE_NAME, ThisFunction );
        }

/*
** Map the PLX control reg address.  If we don't have a PLX device then work
** with the COMM reg.
*/
        if ( PlxFound )
        {
            PlxMappedAddr = (char *)ioremap_nocache( UviPlxDevice->base_address[2],
                                                                             0x0010 );
            if ( PlxMappedAddr == NULL )
            {
                WHENDEBUG( UVI_DBERROR )
                {
                    printk( KERN_ERR "%s:%s; NULL returned from PLX ioremap, this"
                            " is a problem!\n", MODULE_NAME, ThisFunction );
                }
                return( UVI_NULL_PTR );
            }

            WHENDEBUG( UVI_DBEXTRA )
            {
                printk( KERN_INFO "%s:%s; PLX data read is 0x%08x.\n", MODULE_NAME,
                        ThisFunction, *(UVI_UINT16 *)PlxMappedAddr );
            }

            *(UVI_UINT16 *)PlxMappedAddr = MEC_PLX |
                                           SEC_PLX |
                                           ABLE_PLX |
                                           VME_EN_PLX;

            WHENDEBUG( UVI_DBEXTRA )
            {
                printk( KERN_INFO "%s:%s; New PLX data val read is 0x%08x.\n", MODULE_NAME,
                        ThisFunction, *(UVI_UINT16 *)PlxMappedAddr );
            }

/* Write the BERR bit to the berr status reg., to clear it before using. */
            *(UVI_UINT32 *)( PlxMappedAddr + PLX_B_INT_STATUS ) |= BERR_S_PLX;

/*
** Need this ptr in 'uvimod_ops', don't unmap here.
     iounmap( PlxMappedAddr );
     PlxMappedAddr = 0x0;
*/

        }
        else
        {
/* PLX not found, use COMM register. */
            IsaCmdRegVal = readw( NOTPLX_CMDREG_ADDR );
            WHENDEBUG( UVI_DBEXTRA )
            {
                printk( KERN_INFO "%s:%s; ISA command reg initial value is 0x%08x.\n",
                        MODULE_NAME, ThisFunction, IsaCmdRegVal );
            }

            IsaCommandBits = MEC_NOTPLX |
                             SEC_NOTPLX |
                             ABLE_NOTPLX |
                             VME_EN_NOTPLX;

            IsaCmdRegVal = IsaCommandBits;
            writew( IsaCmdRegVal, NOTPLX_CMDREG_ADDR );

            IsaCmdRegVal = readw( NOTPLX_CMDREG_ADDR );
            WHENDEBUG( UVI_DBEXTRA )
            {
                printk( KERN_INFO "%s:%s; ISA command reg value is now 0x%08x.\n",
                        MODULE_NAME, ThisFunction, IsaCmdRegVal );
            }

/* Write the BERR bit to the berr status reg., to clear it before using. */
            IsaCmdRegVal |= 0x80;
            writel( IsaCmdRegVal, NOTPLX_CMDREG_ADDR );
        }

/* Initially clear any active interrupt status. */
        if ( Univ1Found )
        {
            *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) |= LINT_CLR_ALL_U1;
            *(UVI_UINT32 *)( UnivBaseAddr + VINT_STAT ) |= VINT_CLR_ALL_U1;
         }
        else
        {
            *(UVI_UINT32 *)( UnivBaseAddr + LINT_STAT ) |= LINT_CLR_ALL_U2;
            *(UVI_UINT32 *)( UnivBaseAddr + VINT_STAT ) |= VINT_CLR_ALL_U2;
        }

/* Set flag indicating device initialized and all is well. */
        UdeviceOk = UVI_TRUE;
    }
    else
    {
        WHENDEBUG( UVI_DBERROR )
        {
            printk( KERN_ERR "%s:%s; NULL pointer, no access to system PCI devices list,"
            " abort!\n", MODULE_NAME, ThisFunction );
        }
        return( UVI_NULL_PTR );
    }

    WHENDEBUG( UVI_DBTRACE ) printk( KERN_INFO "%s:%s; Exiting ...\n", MODULE_NAME,
                                     ThisFunction );

    return( UVI_SUCCESS );

} /* end 'uviInitDev' */
